#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>
#include <windows.h>

#include "simulator.h"
#include "robot.h"
#include "config.h"
#include "interfaceSimulator.h"


//------------------------------Private functions declarations
static void display();
static void keyboard(unsigned char key, int x, int y);
static void mousePress(int bouton, int state, int x, int y);
static void reshape(int width, int height);
static void mouseMotion(int x, int y);
static void displayRobot();
static void calcCosSinTable();
static void displayWireCircle(int radius);
static void initValues();
static void keyboardSpe(int key, int x, int y);
static void idle();
static void end();

//------------------------------Private attributes

/* General */
static double Sin[360], Cos[360];

/* Camera */
int WORLD_SIZE;
static double ZOOM_MIN;
static double ZOOM_MAX;
static double RX_MIN;
static double RX_MAX;
static double PX_MIN;
static double PX_MAX;
static double PZ_MIN;
static double PZ_MAX;
static double xold, yold, zoom, px, pz;
static int ry;
static int rx;
static unsigned char buttonLeft, buttonRight, buttonMiddle, manual;

/*Light */
GLfloat L0pos[3];
GLfloat L0dir[3]= { 0, 0, 0 };
/* White light */
GLfloat L0dif[]= { 1, 1, 1 };
/* Refelction */
GLfloat Mshiny=120;

//------------------------------Public functions
int initSimulator(int argc, char **argv) {

	/* Initialisation glut and window */
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowPosition(200, 200);
	glutInitWindowSize(500, 500);
	glutCreateWindow("Sumo");

	/* Initialisation scene */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);

	/* Light */
	glShadeModel(GL_SMOOTH);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,L0dif);
	glLightfv(GL_LIGHT0,GL_SPECULAR,L0dif);

	//	glEnable( GL_POLYGON_SMOOTH);
	//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//	glEnable( GL_BLEND);
	//	glEnable(GL_LINE_SMOOTH);

	//Init World
	WORLD_SIZE=2*PLATFORM_RADIUS+WORLD_BORDER;

	manual=0;

	calcCosSinTable();
	initValues();

	/* Initilize completly the robot */
	//Stop robot
	setWheels(STOP,STOP);
	//Speed
	setSpeed(0);
	//Sensors
	setTouch1State(UNPRESSED);
	setTouch3State(UNPRESSED);
	setLight2State(FIELD_BRIGHTNESS);
	//Place the robot
	moveRobotTo(0, ROBOT_START_POSITION, 0);
	
	/* Register callback functions */
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mousePress);
	glutMotionFunc(mouseMotion);
	glutReshapeFunc(reshape);
	glutIdleFunc(idle);
	glutSpecialFunc(keyboardSpe);

	/* let's go */
	glutMainLoop();
	return 0;
}

//------------------------------Private functions
static void end(void) {
	//Kill stimulate
	initialized=0;
	ExitThread(0);
}

static void idle(void) {
	double wheels[2];
	double positionRobot[3];
	double distFromCenter=0;
	double distFromSensor=0;
	double xsensor=0;
	double zsensor=0;

	getPositionRobot(positionRobot);
	getWheels(wheels);

	//Move the robot
	if (!manual) {
		if (wheels[0] == wheels[1]) {
			moveRobotToAdd(Sin[(int)positionRobot[2]]*wheels[0], Cos[(int)positionRobot[2]]*wheels[0], 0);
		} else {
			if (wheels[0] < wheels[1])
				moveRobotToAdd(0, 0, ROTATE_STEP);
			else
				moveRobotToAdd(0, 0, -ROTATE_STEP);
		}
	}

	//Check positions
	getPositionRobot(positionRobot);
	//Pythagore for center of the robot
	distFromCenter=sqrt((positionRobot[0]*positionRobot[0])+(positionRobot[1]
			*positionRobot[1]));
	//Pythagore for light sensor
	xsensor=Sin[(int)positionRobot[2]]*ROBOT_SIZE/2+positionRobot[0];
	zsensor=Cos[(int)positionRobot[2]]*ROBOT_SIZE/2+positionRobot[1];
	distFromSensor=sqrt((xsensor*xsensor)+(zsensor *zsensor));
	//Light Sensor position
	if (distFromSensor>PLATFORM_RADIUS) {
		setLight2State(LINE_BRIGHTNESS);
	} else {
		setLight2State(FIELD_BRIGHTNESS);
	}
	//Gravity Center of the robot
	if (distFromCenter>PLATFORM_RADIUS) {
		printf("Robot went out of the platform\n");
		//end();
	}

	L0pos[0]=Cos[ry]*Cos[rx]*zoom;
	L0pos[1]=Sin[ry]*Cos[rx]*zoom;
	L0pos[2]=Sin[rx]*zoom;

	/*Refresh*/
	glutPostRedisplay();
	/* Sleep a bit the thread otherwise the CPU goes crazy */
	Sleep(IDLE_SLEEP);
}

static void initValues() {
	//Init Cam
	zoom=WORLD_SIZE;
	ZOOM_MIN=ROBOT_SIZE;
	ZOOM_MAX=WORLD_SIZE*2;
	RX_MIN=0.1;
	RX_MAX=90;
	PX_MIN=-WORLD_SIZE;
	PX_MAX=WORLD_SIZE;
	PZ_MIN=-WORLD_SIZE;
	PZ_MAX=WORLD_SIZE;
	rx=45;
	ry=0;
	px=0;
	pz=0;
}

static void reshape(int width, int height) {
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45, (double)width/(double)height, 0.1, ZOOM_MAX*2);
	glMatrixMode(GL_MODELVIEW);
}

static void displayRobot() {
	double positionRobot[3];
	/* White */
	GLfloat robotColor[]= { 1, 1, 1 };
	/* Red */
	GLfloat touchSensorColor[]= { 1, 0, 0 };
	/* Blue */
	GLfloat lightSensorColor[]= { 0, 0, 1 };

	getPositionRobot(positionRobot);

	glTranslated(positionRobot[0], (double)ROBOT_SIZE/(double)2,
			positionRobot[1]);
	glRotated(positionRobot[2], 0, 1, 0);
	glPushMatrix();

	//Global box
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,robotColor);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,robotColor);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,Mshiny);
	glutSolidCube(ROBOT_SIZE);

	glPopMatrix();
	glPushMatrix();

	//Front (light)
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,lightSensorColor);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,lightSensorColor);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,Mshiny);
	glTranslated(0, -(((double)ROBOT_SIZE/(double)2) -((double)SIZE_SENSOR
			/(double)2)), (double)ROBOT_SIZE/(double)2);
	glutSolidCube(SIZE_SENSOR);

	glPopMatrix();
	glPushMatrix();

	//Sensor right
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,touchSensorColor);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,touchSensorColor);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,Mshiny);
	glTranslated((double)-ROBOT_SIZE/(double)2, 0, 0);
	glutSolidCube(SIZE_SENSOR);

	glPopMatrix();

	//Sensor left
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,touchSensorColor);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,touchSensorColor);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,Mshiny);
	glTranslated((double)ROBOT_SIZE/(double)2, 0, 0);
	glutSolidCube(SIZE_SENSOR);
}

static void displayWireCircle(int radius) {
	int i=0;
	double x=0, y=0;

	glBegin(GL_LINE_LOOP);
	for (i=0; i<360; i++) {
		x = radius * Cos[i];
		y= radius * Sin[i];
		glVertex2f(x, y);
	}
	glEnd();
}

static void display() {

	/* Clear background color */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	/* Position of the camera and orientation */
	glTranslated(0, 0, -zoom);
	glRotated(rx, 1, 0, 0);
	glTranslated(px, 0, pz);
	glRotated(ry, 0, 1, 0);

	/* Objects */
	/* World */
	glPushMatrix();
	glColor3f(1, 1, 1);
	glTranslated(0, 0.25*WORLD_SIZE/2, 0);
	glScalef(1.0, 0.25, 1.0);
	glutWireCube(WORLD_SIZE);
	glPopMatrix();

	/* Platform */
	glPushMatrix();
	glColor3f(0, 1, 0);
	glRotated(90, 1, 0, 0);
	displayWireCircle(PLATFORM_RADIUS);
	glPopMatrix();

	/* Robot */
	/* Only the robot has light*/
	glEnable(GL_LIGHTING);
	displayRobot();
	glDisable(GL_LIGHTING);

	/* Force to print */
	glFlush();
	/* Double buffer */
	glutSwapBuffers();

}

static void keyboardSpe(int key, int x, int y) {
	switch (key) {
	case GLUT_KEY_LEFT:
		moveRobotToAdd(STEP*DEFAULT_SPEED, 0,0);
		break;
	case GLUT_KEY_RIGHT:
		moveRobotToAdd(-STEP*DEFAULT_SPEED, 0, 0);
		break;
	case GLUT_KEY_UP:
		moveRobotToAdd(0, STEP*DEFAULT_SPEED, 0);
		break;
	case GLUT_KEY_DOWN:
		moveRobotToAdd(0, -STEP*DEFAULT_SPEED, 0);
		break;
	case GLUT_KEY_PAGE_UP:
		moveRobotToAdd(0, 0, ROTATE_STEP);
		break;
	case GLUT_KEY_PAGE_DOWN:
		moveRobotToAdd(0, 0, -ROTATE_STEP);
		break;
	}
}

static void keyboard(unsigned char key, int x, int y) {
	switch (key) {
	case 'i':
		//Reset Camera
		initValues();
		glutPostRedisplay();
		break;
	case 'm':
		//Send touch1
		manual=!manual;
	case '1':
		//Send touch1
		if (getTouch1State()==PRESSED)
			setTouch1State(UNPRESSED);
		else
			setTouch1State(PRESSED);
		break;
	case '2':
		//Send touch3
		if (getTouch3State()==PRESSED)
			setTouch3State(UNPRESSED);
		else
			setTouch3State(PRESSED);
		break;
	case 'q':
		end();
	}
}

static void mousePress(int button, int state, int x, int y) {
	if (button==GLUT_LEFT_BUTTON&&state==GLUT_DOWN) {
		xold=x;
		yold=y;
		buttonLeft=PRESSED;
	} else {
		buttonLeft=UNPRESSED;
	}
	if (button==GLUT_RIGHT_BUTTON&&state==GLUT_DOWN) {
		xold=x;
		yold=y;
		buttonRight=PRESSED;
	} else {
		buttonRight=UNPRESSED;
	}
	if (button==GLUT_MIDDLE_BUTTON&&state==GLUT_DOWN) {
		xold=x;
		yold=y;
		buttonMiddle=PRESSED;
	} else {
		buttonMiddle=UNPRESSED;
	}
}

static void mouseMotion(int x, int y) {
	if (buttonLeft && !buttonRight) {
		ry+=x-xold;
		if (ry>=360)
			ry-=360;
		if (ry<0)
			ry=360+ry;
		xold=x;

		rx+=y-yold;
		if (rx>=RX_MAX)
			rx=RX_MAX;
		if (rx<RX_MIN)
			rx=RX_MIN;
		yold=y;
	}

	if (buttonRight && !buttonLeft) {
		zoom+=y-yold;
		if (zoom>=ZOOM_MAX)
			zoom=ZOOM_MAX;
		if (zoom<ZOOM_MIN)
			zoom=ZOOM_MIN;
		yold=y;
	}

	if (buttonMiddle) {
		px+=x-xold;
		if (px>=PX_MAX)
			px=PX_MAX;
		if (px<PX_MIN)
			px=PX_MIN;
		xold=x;

		pz+=y-yold;
		if (pz>=PZ_MAX)
			pz=PZ_MAX;
		if (pz<PZ_MIN)
			pz=PZ_MIN;
		yold=y;
	}

	L0pos[0]=Sin[ry]*Cos[rx]*zoom;
	L0pos[1]=Cos[ry]*Cos[rx]*zoom;
	L0pos[2]=Sin[rx]*zoom;

	glutPostRedisplay();
}

static void calcCosSinTable() {
	int i;
	for (i=0; i<360; i++) {
		Sin[i]=sin(M_PI*i/180);
		Cos[i]=cos(M_PI*i/180);
	}
}
