/*
 * File: sys_user_co.c
 *
 * Description:
 * Function definitions for user supplied (non-translated) call out entry
 * points in the generated application.
 *
 *
 * Notice:
 * (C) Copyright 1999-2002 ROX Software, Inc.
 *     All rights reserved.
 * Enhancements provided by TOYO Corporation.
 *
 * This document contains confidential and proprietary information and
 * property of ROX Software, Inc.  No part of this document may be
 * reproduced without the express written permission of ROX Software, Inc.
 */

#include <conio.h>
#include <unistd.h>
#include <interfaceSimulator.h>
#include "sys_user_co.h"
#include "S_sync_services.h"

 /*
 * UserInitializationCallout
 *
 * This function is invoked at the immediate begining of application
 * initialization. It is the very first function to be executed at system
 * startup.
 * User supplied implementation of this function should be restricted to
 * things like memory initialization, early hardware duties, etc.
 *
 */
void UserInitializationCalloutf()
{
  TIM_init();
}

/*
 * UserPreOoaInitializationCallout
 *
 * This function is invoked immediately prior to executing any OOA application
 * analysis initialization objects.
 *
 */
void UserPreOoaInitializationCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserPostOoaInitializationCallout
 *
 * This function is invoked immediately after executing any OOA application
 * analysis initialization objects.
 * When this callout function returns, the system dispatcher will allow the
 * OOA application analysis state models to start consuming events.
 *
 */
void UserPostOoaInitializationCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserBackgroundProcessingCallout
 *
 * This function is invoked once during each loop execution of the system
 * dispather.
 * It is invoked at the 'top' of the system dispatcher loop, immediately
 * prior to dispatching any OOA application analysis events.
 *
 */
void UserBackgroundProcessingCalloutf()
{
  static bool old_1 = false;
  static unsigned char old_2 = 0, i = 0;
  static bool old_3 = false;
  bool one, three;
  unsigned char two;

  /*------------------------------------------------------------------------*/
  /* Read a sensor.  Determine if it has changed since the last read.       */
  /* If so, notify the application.                                         */
  /* Touch sensors are checked ahead of the light sensor.                   */
  /*------------------------------------------------------------------------*/
  one = touch1();
  two = light2();
  three = touch3();
  if ( one != old_1 ) {
    old_1 = one;
    S_touch1_sync( one );
  } else if ( three != old_3 ) {
    old_3 = three;
    S_touch3_sync( three );
  } else if ( two != old_2 ) {
    old_2 = two;
    S_light2_sync( two );
  }
  TIM_tick();
  /* Comment this display command out if it becomes too burdensome.  */
  //if ( i++ == 0 ) { /* Refresh display once every 256 times around.  */
  //  cputw( ( one << 12 ) + ( two << 4 ) + three );
  //}
}

/*
 * UserPreShutdownCallout
 *
 * This function is invoked at termination of the system dispatcher, but
 * prior to performing any OOA application analysis shutdown sequencing.
 *
 */
void UserPreShutdownCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserPostShutdownCallout
 *
 * This function is invoked immediately before application exit.
 *
 */
void UserPostShutdownCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserEventCantHappenCallout
 *
 * This function is invoked any time that an event is received that
 * results in a "cant happen" transition.
 *
 */
void UserEventCantHappenCalloutf( const Escher_StateNumber_t current_state,
                                  const Escher_StateNumber_t next_state,
                                  const Escher_EventNumber_t event_number )
{
  /* Insert implementation specific code here.  */
}

/*
 * UserEventNoInstanceCallout
 *
 * This function is invoked when we failed to validate the instance
 * to which an event was directed.  This can happen in normal operation
 * when the instance was deleted while the event was in flight (analysis
 * error).
 *
 */
void UserEventNoInstanceCalloutf( const Escher_EventNumber_t event_number )
{
  /* Insert implementation specific code here.  */
}

/*
 * UserEventFreeListEmptyCallout
 *
 * This function is invoked when an attempt is made to allocate an
 * event, but there are no more left.
 *
 */
void UserEventFreeListEmptyCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserEmptyHandleDetectedCallout
 *
 * This function is invoked when an attempt is made to use an instance
 * reference variable (handle) that is null (empty).
 *
 */
void UserEmptyHandleDetectedCalloutf( const char * object_keyletters,
                                      const char * s )
{
  /* Insert implementation specific code here.  */
}

/*
 * UserObjectPoolEmptyCallout
 *
 * This function is invoked when an attempt is made to create an
 * instance of an object, but there are no instances available.
 *
 */
void UserObjectPoolEmptyCalloutf( const char * domain,
                                  const char * object_name )
{
  /* Insert implementation specific code here.  */
}

/*
 * UserNodeListEmptyCallout
 *
 * This function is invoked when an attempt is made to allocate a
 * node, but there are no more left.
 *
 */
void UserNodeListEmptyCalloutf()
{
  /* Insert implementation specific code here.  */
}

/*
 * UserInterleavedBridgeOverflowCallout
 *
 * This function is invoked when an attempt is made to post too many
 * interleaved bridges.  The depth of this list is defined by
 * SYS_MAX_INTERLEAVED_BRIDGES (unless changed in the archetype).
 *
 */
void UserInterleavedBridgeOverflowCalloutf()
{
  /* Insert implementation specific code here.  */
}
